@@ -18,21 +18,53 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "notebook_data.h"
-#include <glom/libglom/data_structure/glomconversions.h>
+#include <glom/mode_data/notebook_data.h>
+#include <libglom/data_structure/glomconversions.h>
 #include <glibmm/i18n.h>
 
+#ifdef GLOM_ENABLE_MAEMO
+#include <hildonmm/program.h>
+#endif
+
 namespace Glom
 {
 
 Notebook_Data::Notebook_Data()
+: 
+  #ifdef GLOM_ENABLE_MAEMO
+  m_window_maemo_details(0),
+  #endif
+  m_iPage_Details(0), m_iPage_List(0)
 {
   //Add Pages:
   pages().push_back(Gtk::Notebook_Helpers::TabElem(m_Box_List, _("List")));
   m_iPage_List = 0;
 
+  #ifndef GLOM_ENABLE_MAEMO
   pages().push_back(Gtk::Notebook_Helpers::TabElem(m_Box_Details, _("Details")));
   m_iPage_Details = 1;
+  #else
+  //On Maemo, we add the box to m_window_maemo_details instead:
+  m_window_maemo_details = new Window_BoxHolder(&m_Box_Details, _("Details"));
+  
+  //Let this window have the main AppMenu:
+  Hildon::Program::get_instance()->add_window(*m_window_maemo_details);
+  
+  Gtk::Window* pWindow = get_app_window();
+  if(pWindow)
+    m_window_maemo_details->set_transient_for(*pWindow);
+    
+  //Refresh the list when the details window is closed:
+  m_window_maemo_details->signal_hide().connect(
+    sigc::mem_fun(*this, &Notebook_Data::on_window_maemo_details_closed));
+
+  m_Box_Details.show_all();
+  #endif //GLOM_ENABLE_MAEMO
+
+  // Set accessible name for the notebook, to be able to access it via LDTP
+#ifdef GTKMM_ATKMM_ENABLED
+  get_accessible()->set_name(_("List Or Details View"));
+#endif  
 
 
   //Connect signals:
@@ -41,6 +73,7 @@ Notebook_Data::Notebook_Data()
   m_Box_List.signal_user_requested_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_list_user_requested_details));
 
   //Allow Details to ask List to ask Details to show a different record:
+  #ifndef GLOM_ENABLE_MAEMO //These navigation buttons are not visible on Maemo.
   m_Box_Details.signal_nav_first().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_nav_first));
   m_Box_Details.signal_nav_prev().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_nav_previous));
   m_Box_Details.signal_nav_next().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_nav_next));
@@ -48,7 +81,8 @@ Notebook_Data::Notebook_Data()
 
   //Allow Details to tell List about record deletion:
   m_Box_Details.signal_record_deleted().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_record_deleted));
-
+  #endif //GLOM_ENABLE_MAEMO
+  
   //Allow Details to ask to show a different record in a different table:
   m_Box_Details.signal_requested_related_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_details_user_requested_related_details));
 
@@ -62,14 +96,34 @@ Notebook_Data::Notebook_Data()
   //This is hidden by default,
   m_Box_Details.show_layout_toolbar(false);
 #endif //GLOM_ENABLE_CLIENT_ONLY
+
+#ifdef GLOM_ENABLE_MAEMO
+  //On Maemo we show the details in a separate window.
+  set_show_tabs(false);
+#endif //GLOM_ENABLE_MAEMO
 }
 
 Notebook_Data::~Notebook_Data()
 {
   remove_view(&m_Box_List);
   remove_view(&m_Box_Details);
+
+#ifdef GLOM_ENABLE_MAEMO
+  if(m_window_maemo_details)
+    delete m_window_maemo_details;
+#endif //GLOM_ENABLE_MAEMO
 }
 
+#ifdef GLOM_ENABLE_MAEMO
+void Notebook_Data::on_window_maemo_details_closed()
+{
+  //Show the changed record in the list:
+  const Gnome::Gda::Value primary_key_selected = m_Box_List.get_primary_key_value_selected();
+  m_Box_List.refresh_data_from_database();
+  m_Box_List.set_primary_key_value_selected(primary_key_selected);
+}
+#endif //GLOM_ENABLE_MAEMO
+
 bool Notebook_Data::init_db_details(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value_for_details)
 {
   m_table_name = found_set.m_table_name;
@@ -91,67 +145,52 @@ bool Notebook_Data::init_db_details(const FoundSet& found_set, const Gnome::Gda:
     // Ignore error, sharedconnection is not used directly within this function
 #endif
 
-    //const FoundSet old_found_set = m_Box_List.get_found_set();
-    //std::cout << "  old_where_clause=" << old_where_clause << std::endl;
-    //std::cout << "  where_clause=" << where_clause << std::endl;
-    //const bool new_find_set = !(found_set == old_found_set);
+    result = m_Box_List.init_db_details(found_set, get_active_layout_platform(get_document())); //TODO: Select the last selected record.
 
-    //if(get_current_view() == DATA_VIEW_List)
-    //{
-      result = m_Box_List.init_db_details(found_set); //TODO: Select the last selected record.
-      //m_Box_List.load_from_document();
-    //}
-    //else //DATA_VIEW_Details
-    //{
-      //Show the previously-shown record, if there is one, if this is not a new found-set (via a new where_clause)
-      //so that returning to this table will return the user to the same record:
-      Document_Glom* document = get_document();
-      if(document)
+    //Show the previously-shown record, if there is one, if this is not a new found-set (via a new where_clause)
+    //so that returning to this table will return the user to the same record:
+    Document* document = get_document();
+    if(document)
+    {
+      Gnome::Gda::Value primary_key_for_details;
+
+      if(!details_record_specified)
       {
-        Gnome::Gda::Value primary_key_for_details;
-
-        if(!details_record_specified)
-        {
-          //std::cout << "debug: no new_found_set" << std::endl;
-          primary_key_for_details = document->get_layout_record_viewed(m_table_name, m_Box_Details.get_layout_name());
-        }
-        else if(details_record_specified)
-        {
-          primary_key_for_details = primary_key_value_for_details;
-        }
-        else
-        {
-          //std::cout << "debug: new_found_set" << std::endl;
-        }
-
-
-        //If the specified (or remembered) primary key value is not in the found set, 
-        //then ignore it:
-        if(!found_set.m_where_clause.empty() && !get_primary_key_is_in_foundset(found_set, primary_key_for_details))
-        {
-          primary_key_for_details = Gnome::Gda::Value(); //TODO: We set it to empty just so we can test if for empty.
-        }
+        //std::cout << "debug: no new_found_set" << std::endl;
+        primary_key_for_details = document->get_layout_record_viewed(m_table_name, m_Box_Details.get_layout_name());
+      }
+      else if(details_record_specified)
+      {
+        primary_key_for_details = primary_key_value_for_details;
+      }
+      
+      //If the specified (or remembered) primary key value is not in the found set, 
+      //then ignore it:
+      if(!found_set.m_where_clause.empty() && !get_primary_key_is_in_foundset(found_set, primary_key_for_details))
+      {
+        primary_key_for_details = Gnome::Gda::Value(); //TODO: We set it to empty just so we can test if for empty.
+      }
 
+      if(Conversions::value_is_empty(primary_key_for_details))
+      {
+        //Make sure that the details view is not empty, if there are any records to show:
+        primary_key_for_details = m_Box_List.get_primary_key_value_selected();
+        //std::cout << "debug:  m_Box_List.get_primary_key_value_selected()=" << primary_key_for_details.to_string() << std::endl;
         if(Conversions::value_is_empty(primary_key_for_details))
         {
-          //Make sure that the details view is not empty, if there are any records to show:
-          primary_key_for_details = m_Box_List.get_primary_key_value_selected();
-          //std::cout << "debug:  m_Box_List.get_primary_key_value_selected()=" << primary_key_for_details.to_string() << std::endl;
-          if(Conversions::value_is_empty(primary_key_for_details))
-          {
-            //std::cout << "debug: calling list.get_primary_key_value_first()" << std::endl;
-            primary_key_for_details = m_Box_List.get_primary_key_value_first();
-            //std::cout << "  debug:  result=" <<  primary_key_for_details.to_string() << std::endl;
-          }
+          //std::cout << "debug: calling list.get_primary_key_value_first()" << std::endl;
+          primary_key_for_details = m_Box_List.get_primary_key_value_first();
+          //std::cout << "  debug:  result=" <<  primary_key_for_details.to_string() << std::endl;
         }
-
-        m_Box_Details.init_db_details(found_set, primary_key_for_details);
       }
-      else
-        std::cerr << "Notebook_Data::init_db_details(): document is NULL" << std::endl;
-    //}
+
+      m_Box_Details.init_db_details(found_set, get_active_layout_platform(get_document()), primary_key_for_details);
+    }
+    else
+      std::cerr << "Notebook_Data::init_db_details(): document is NULL" << std::endl;
   }
 
+
   //Block this handler temporarily because we don't need another refresh from the database:
   if(m_connection_switch_page)
     m_connection_switch_page.block();
@@ -170,7 +209,7 @@ bool Notebook_Data::init_db_details(const FoundSet& found_set, const Gnome::Gda:
     Glib::ustring current_layout;
     if(!details_record_specified)
     {
-      Document_Glom* document = get_document(); 
+      Document* document = get_document(); 
       if(document)
         current_layout = document->get_layout_current(m_table_name);
     }
@@ -194,26 +233,52 @@ FoundSet Notebook_Data::get_found_set() const
   return m_Box_List.get_found_set();
 }
 
-FoundSet Notebook_Data::get_found_set_details() const
-{
-  return m_Box_Details.get_found_set();
-}
-  
-
-void Notebook_Data::on_list_user_requested_details(const Gnome::Gda::Value& primary_key_value)
+void Notebook_Data::show_details(const Gnome::Gda::Value& primary_key_value)
 {
   //Prevent n_switch_page_handler() from doing the same thing:
   if(m_connection_switch_page)
     m_connection_switch_page.block();
-    
-  m_Box_Details.refresh_data_from_database_with_primary_key(primary_key_value);
   
+  //std::cout << "DEBUG: Notebook_Data::show_details() primary_key_value=" << primary_key_value.to_string() << std::endl;
+  m_Box_Details.refresh_data_from_database_with_primary_key(primary_key_value);
+
+#if GLOM_ENABLE_MAEMO
+  //Details are shown in a separate window on Maemo,
+  //though that window contains the regular m_Box_Details. 
+  //TODO: Use the singular form when it's available from the document.
+  Document* document = get_document();
+  g_assert(document);
+  const Glib::ustring title = 
+    Glib::ustring::compose(_("%1 Details"), 
+      document->get_table_title_singular(m_table_name));
+  m_window_maemo_details->set_title(title);
+  
+  m_window_maemo_details->show();
+#else  
   if(get_current_view() != DATA_VIEW_Details)
     set_current_view(DATA_VIEW_Details);
+#endif
   
   //Re-enable this handler, so we can respond to notebook page changes:
   if(m_connection_switch_page)
-    m_connection_switch_page.unblock();
+    m_connection_switch_page.unblock();  
+}
+
+bool Notebook_Data::on_idle_show_details(const Gnome::Gda::Value& primary_key_value)
+{
+  show_details(primary_key_value);
+  return false; //Don't call this idle handler again.
+}
+
+void Notebook_Data::on_list_user_requested_details(const Gnome::Gda::Value& primary_key_value)
+{
+  //Show the details after a delay,
+  //to avoid problems with deleting the list GtkCellRenderer while 
+  //handling its signal.
+  Glib::signal_idle().connect(
+    sigc::bind(
+      sigc::mem_fun(*this, &Notebook_Data::on_idle_show_details),
+      primary_key_value));
 }
 
 void Notebook_Data::on_details_user_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
@@ -229,6 +294,12 @@ void Notebook_Data::on_details_user_requested_related_details(const Glib::ustrin
   */
 }
 
+FoundSet Notebook_Data::get_found_set_details() const
+{
+  return m_Box_Details.get_found_set();
+}
+
+
 void Notebook_Data::set_current_view(dataview view)
 {
   if(view == DATA_VIEW_List)
@@ -305,12 +376,6 @@ Notebook_Data::type_signal_record_details_requested Notebook_Data::signal_record
   return m_signal_record_details_requested;
 }
 
-void Notebook_Data::show_details(const Gnome::Gda::Value& primary_key_value)
-{
-  //Reuse this implementation:
-  on_list_user_requested_details(primary_key_value);
-}
-
 void Notebook_Data::on_switch_page_handler(GtkNotebookPage* pPage, guint uiPageNumber)
 {
   //Call base class:
@@ -320,7 +385,7 @@ void Notebook_Data::on_switch_page_handler(GtkNotebookPage* pPage, guint uiPageN
   Box_Data* box = dynamic_cast<Box_Data*>(get_nth_page(uiPageNumber));
   if(box)
   {
-    Document_Glom* document = get_document();
+    Document* document = get_document();
     if(document)
       document->set_layout_current(m_table_name, box->get_layout_name());
 
@@ -349,4 +414,12 @@ void Notebook_Data::get_record_counts(gulong& total, gulong& found)
   m_Box_List.get_record_counts(total, found);
 }
 
+#ifdef GLOM_ENABLE_MAEMO
+void Notebook_Data::do_menu_file_add_record()
+{
+  show_details(Gnome::Gda::Value());
+  m_Box_Details.do_new_record();
+}
+#endif //GLOM_ENABLE_MAEMO
+
 } //namespace Glom
